{-|
Module      : Main
Description : EPO Patents database crawler (API client)
Copyright   : (c) CNRS, 2023-present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX
-}


module Main where

import Data.Text qualified as T
import EPO.API.Client qualified as Client
import EPO.API.Client.Implementation (searchEPOAPI)
import EPO.API.Client.Types
import Network.URI
import Options.Applicative.Simple
import Protolude hiding (option)
import System.IO.Error (userError)
import Text.Pretty.Simple (pPrint)

data AuthOptions = AuthOptions
  { user  :: Text
  , token :: Text
  } deriving (Eq, Show)

data SearchOptions = SearchOptions
  { authOptions :: AuthOptions
  , apiUrl      :: Text
  , start       :: Maybe Start
  , end         :: Maybe End
  , search      :: Search
  } deriving (Eq, Show)

main :: IO ()
main = do
  let authOptions =
        AuthOptions <$>
          strOption (long "user") <*>
          strOption (long "token")
  let searchOptions =
        SearchOptions <$>
          authOptions <*>
          (strOption (long "api-url")) <*>
          optional (option auto (long "start")) <*>
          optional (option auto (long "end")) <*>
          (strOption (long "search"))

  (opts, runCmd) <-
    simpleOptions "0.1.0.0"
                  "epo-api-client"
                  "epo-api-client command line"
                  (pure ()) $ do
      addCommand "search"
        "Search epo published data (via API)"
        searchPublishedData
        searchOptions

  runCmd ()

searchPublishedData :: SearchOptions -> () -> IO ()
searchPublishedData opts@SearchOptions { .. } _ = do
  putText $ "Options: " <> show opts
  let rStart = fromMaybe 0 start
  let rEnd = fromMaybe 20 end
  case parseURI (T.unpack apiUrl) of
    Nothing -> ioError $ userError $ T.unpack $ "Error parsing api url " <> apiUrl
    Just uri -> do
      Paginated { .. } <- searchEPOAPI uri (authKeyFromAuthOptions authOptions) rStart rEnd search
      mapM_ pPrint items


authKeyFromAuthOptions :: AuthOptions -> AuthKey
authKeyFromAuthOptions AuthOptions { .. } =
  AuthKey { user  = User user
          , token = Token token }
